#ifndef CG_MANAGER_H
#define CG_MANAGER_H

#include <GL/glew.h>
#include <Cg/Cg.h>
#include <Cg/cgGL.h>

#include <string>
#include <map>

#include "render_buffer.h"

/*****************************************************************************/

class CG_Program
{
private:
	CGprogram program_;
	CGprofile profile_;
	
public:
	CGparameter getParameter(const std::string& paramName);
	void setParameter(const std::string& paramName, float x);
	void setParameter(const std::string& paramName, float x, float y);
	void setParameter(const std::string& paramName, float x, float y, float z);
	void setParameter(const std::string& paramName, float x, float y, float z, float w);
	void setParameter(const std::string& paramName, CGGLenum matrix, CGGLenum xform);

	//void setParameter(const std::string& paramName, const Render_Buffer& textureId);

	void   setTexture(const std::string& paramName, const Render_Buffer& texture);
	void   setTexture(const std::string& paramName, GLuint texture);
	void unsetTexture(const std::string& paramName);

	void enable();
	void disable();

	CG_Program(CGcontext context,
		   const std::string& fileName,
		   const std::string& procedureName,
		   CGenum programType,
		   CGprofile profile);

	CG_Program(const CG_Program& o): program_(o.program_), profile_(o.profile_) {};
	CG_Program() {};
		
};

class CG_Manager
{
private:
	CGcontext context_;
	std::map<std::string, CG_Program> programs_;
	static CG_Manager* instance_;

	CG_Manager() 
	{
		context_ = cgCreateContext();
	}

	static CG_Manager &getInstance();

public:
	static void loadProgram(const std::string& fileName,
				const std::string& procedureName,
				const std::string& programName,
				CGprofile profile);

	static void loadAsmProgram(const std::string& fileName,
				const std::string& procedureName,
				const std::string& programName,
				CGprofile profile);

	static CG_Program getProgram(const std::string& programName);
};

#endif