#include <GL/glut.h>
#include <map>

/***************************************************************************/

class Glut_Callbacks
{
	bool hasKey_, hasDisplay_;
	void (*displayCallback_)(void);
	void (*keyCallback_)(unsigned char, int, int);

public:
	Glut_Callbacks(): hasKey_(false), hasDisplay_(false) {}

	void setDisplayCallback(void (*displayCallback)(void)) {
		hasDisplay_ = true;
		displayCallback_ = displayCallback;
	}

	void setKeyboardCallback(void (*keyboardCallback)(unsigned char, int, int)) {
		hasKey_ = true;
		keyCallback_ = keyboardCallback;
	}

	void setGlutCallbacks()
	{
		if (hasKey_)
			glutKeyboardFunc(keyCallback_);
		if (hasDisplay_)
			glutDisplayFunc(displayCallback_);
	}
};

/***************************************************************************/

class Glut_Application
{
private:
	std::map<int, Glut_Callbacks> windows;

	static const int defaultWidth = 640;
	static const int defaultHeight = 480;
	static const int defaultX = 0;
	static const int defaultY = 0;

	Glut_Application()
	{
		int argc = 1;
		char title[100] = "PGHFlow";
		char *argv[1] = {title};
		glutInit(&argc, argv);
	}

	static Glut_Application *instance_;

public:
	int addWindow(int x, int y, int width, int height,
				  std::string windowName, Glut_Callbacks cb);
	int addWindow(int x, int y, 
				  std::string windowName, Glut_Callbacks cb);
	int addWindow(std::string windowName, Glut_Callbacks cb);

	void run();

	static Glut_Application &get_instance()
	{
		if (!instance_)
			instance_ = new Glut_Application();
		return *instance_;
	}
};

