#ifndef RENDER_BUFFER_H
#define RENDER_BUFFER_H

#include "pbuffer.h"
#include "utils.h"
#include <string>
#include <iostream>
#include <cassert>

typedef enum {
	SURFACE_FRONT = WGL_FRONT_LEFT_ARB,
	SURFACE_BACK = WGL_BACK_LEFT_ARB
} Surface_Type;

GLenum toGLenum(Surface_Type);
Surface_Type fromGLenum(GLenum);

class Render_Buffer: public PBuffer
{
private:
	GLuint textureId_;
	Surface_Type bound_;

public:
	Render_Buffer(std::string mode,
				  int width,
				  int height,
				  bool filter):
		PBuffer(mode.c_str())
	{
		check_errors("entering render_buffer ctor");
		Initialize(width, height, false, true);
		Activate();
		glDrawBuffer(GL_FRONT_AND_BACK);

		glDisable(GL_DITHER);
		glDisable(GL_DEPTH_TEST);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0.0, width,
				0.0, height,
				-1.0, 1.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glViewport(0, 0, width, height);

		glGenTextures(1, &textureId_);

		glEnable(GL_TEXTURE_RECTANGLE_NV);
		if (filter) {
			glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		} else {
			glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		}
		glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_RECTANGLE_NV, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		glDrawBuffer(GL_FRONT_AND_BACK);
		glClearColor(0, 0, 0, 0);
		glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
		glEnable(GL_TEXTURE_RECTANGLE_NV);
		glFlush();
		Deactivate();

		std::cout << "Created " << width << "x" << height << " renderbuffer" << std::endl;
		check_errors("leaving render_buffer ctor");
	};

	GLuint getTextureId() const { return textureId_; };

	int Bind(Surface_Type front_surface = SURFACE_FRONT) { 
		glBindTexture(GL_TEXTURE_RECTANGLE_NV, textureId_);
		bound_ = front_surface;
		return PBuffer::Bind(front_surface); // HOHOHO ENTENDI!!
	};

	int Release() {
		return PBuffer::Release(bound_); 
	};

	void set_active_surface(Surface_Type front_surface) {
		check_gl_errors("before void set_active_surface(Surface_Type front_surface)");
		if (front_surface == SURFACE_FRONT)
			glDrawBuffer(GL_FRONT);
		else
			glDrawBuffer(GL_BACK);
		check_gl_errors("after void set_active_surface(Surface_Type front_surface)");
	}

	void draw_on_frame_buffer(int component = - 1, Surface_Type surface = SURFACE_FRONT, bool lerp = false);
	void draw_as_many_components(Surface_Type surface = SURFACE_FRONT);

	void display_buffer(int comps, GLenum surface = GL_FRONT_LEFT);
	void dump_to_stream(std::ostream &os, int comps, GLenum surface = GL_FRONT_LEFT);
};


#endif