#ifndef S_NAVIER_STOKES_H
#define S_NAVIER_STOKES_H

#include "render_buffer.h"
#include "stopwatch.hpp"

const int max_buffer_size = 128;
typedef float SBUFFER[max_buffer_size][max_buffer_size];

class Software_Navier_Stokes_Solver
{
public:
	int domainWidth, domainHeight;
	int imax, jmax;
	Stopwatch timer;

	float reynoldsNumber, dt;
	float U[max_buffer_size][max_buffer_size], 
		V[max_buffer_size][max_buffer_size], 
		F[max_buffer_size][max_buffer_size], 
		G[max_buffer_size][max_buffer_size], 
		P[max_buffer_size][max_buffer_size],
		P2[max_buffer_size][max_buffer_size],
		R[max_buffer_size][max_buffer_size];

	Surface_Type pressure_s1, pressure_s2;

	float dx, dy;

	void init_buffers()
	{
		for (int i=0; i<128; i++)
			for (int j=0; j<128; j++) {
				U[i][j] = 0.0;
				V[i][j] = 0.0;
				F[i][j] = 0.0;
				G[i][j] = 0.0;
				P[i][j] = 0.0;
				P2[i][j] = 0.0;
				R[i][j] = 0.0;
			}
	}

	Software_Navier_Stokes_Solver(int w, int h, float r):
		imax(w), jmax(h),
		domainWidth(w+2), domainHeight(h+2), reynoldsNumber(r) {
			init_buffers();
			dx = 1.0f / domainWidth;
			dy = 1.0f / domainHeight;
			dt = 0.01f;
			pressure_s1 = SURFACE_FRONT;
			pressure_s2 = SURFACE_BACK;
			timer.Start();
			timer.Reset();
		}


	void noslip_south(int from, int to, int y);
	void noslip_north(int from, int to, int y);
	void noslip_east (int from, int to, int x);
	void noslip_west (int from, int to, int x);

	void inflow_south(int from, int to, int y, float fx, float fy);
	void inflow_north(int from, int to, int y, float fx, float fy);
	void inflow_east (int from, int to, int y, float fx, float fy);
	void inflow_west (int from, int to, int y, float fx, float fy);

	void set_uv_boundary();
	void compute_fg();
	void solve_pressure_step(SBUFFER from, SBUFFER to);

	void solve_pressure();
	float get_residual(SBUFFER b);

	void compute_new_uv();
	void adjust_time_step();

	void display_buffer(int b);
	void show_buffer(SBUFFER b);
};

#endif
